#import "Global.h"
#import "GraphView.h"

@implementation GraphView

@synthesize currentDate;
@synthesize birthDate;

- (id)initWithFrame:(CGRect)frame {
	if (self = [super initWithFrame:frame]) {
	}
	return self;
}

- (void)awakeFromNib {
	currentDate = [[NSDateComponents alloc] init];
	
	/* 배경 이미지로 사용할 bio.png 로드 */
	backgroundImage = [[UIImage alloc] initWithContentsOfFile:
					   [[NSBundle mainBundle] pathForResource:@"bio" ofType:@"png"]];
}

- (void)drawRect:(CGRect)rect {
#define START_X			17	// 그래프 시작 X 위치
#define START_Y			22  // 그래프 시작 Y 위치
#define DAY_WIDTH		10  // 일별 간격
#define BOTTOM_HEIGHT	4   // 하단 공백 높이

	/* 데이터별 색상 테이블 */
	static const CGFloat color[MAX_DATATYPE][5] = {
		{ 1.0, 0.0, 0.0, 1.0 }, //red
		{ 0.0, 1.0, 0.0, 1.0 }, //green
		{ 0.0, 0.0, 1.0, 1.0 }  //blue
	};
	
	CGSize viewSize = rect.size;
	CGContextRef context = UIGraphicsGetCurrentContext();

	/* 배경 이미지 출력 */
	CGPoint point;
	point.x = point.y = 0;
	[backgroundImage drawAtPoint:point];

	/* 오늘 날짜를 가르키는 위치는 노란 선으로 출력 */
	CGContextBeginPath(context);
	CGContextSetLineWidth(context, 1.0);
	CGContextSetRGBStrokeColor(context, 1.0, 0.8, 0.0, 1.0);
	CGContextMoveToPoint(context, START_X + (DAY_WIDTH * ([currentDate day] - 1)), START_Y);
	CGContextAddLineToPoint(context, START_X + (DAY_WIDTH * ([currentDate day] - 1)), viewSize.height - BOTTOM_HEIGHT);
	CGContextStrokePath(context);
	
	/* 바이오리듬 출력 */
	int i, j;
	int lineX = START_X;
	
	int vcenter = START_Y + (viewSize.height - START_Y - BOTTOM_HEIGHT)/2; // 그래프의 Y 중간 좌표
	
	/* 바이오리듬 출력 */
	for (i = 1; i < 31; i++) {
		for(j = 0; j < MAX_DATATYPE; j++) {
			CGContextBeginPath(context);
			CGContextSetLineWidth(context, 2.0);
			
			CGContextSetStrokeColor(context, color[j]);
			
			CGContextMoveToPoint(context, lineX, vcenter - biorhythmData[i-1][j]);
			CGContextAddLineToPoint(context, lineX + DAY_WIDTH, vcenter - biorhythmData[i][j]);
			CGContextStrokePath(context);
		}
		lineX += DAY_WIDTH;
	}
}

- (void)dealloc {
	[backgroundImage release];
	[currentDate release];
	[super dealloc];
}

- (void)resetBiorhythmData {
	static const double s_values[MAX_DATATYPE] = {
        23.0, 33.0, 28.0 // 신체, 지성, 감성
    }; 
	
	/* 현재 설정된 날짜의 1일 부터 생일 사이의 날수를 구한다. */ 
	NSDateComponents *firstDate = [[NSDateComponents alloc] init];
	[firstDate setYear: [currentDate year]];
	[firstDate setMonth: [currentDate month]];
	[firstDate setDay:1];
	
	NSDate *tempDay = [[NSCalendar currentCalendar] dateFromComponents:firstDate];
	NSDate *birthDay = [[NSCalendar currentCalendar] dateFromComponents:birthDate];
	
	NSTimeInterval ti = [tempDay timeIntervalSinceDate:birthDay];
	[firstDate release];

	/* 바이오리듬 데이터를 구한다 */
    double days = ceil(fabs(ti) / ONEDAY_SECOND);
	
	for (int i = 0; i < 31; i++) {
		for(int j = 0; j < MAX_DATATYPE; j++) {
			biorhythmData[i][j] = sin((days/s_values[j]) * 2 * 3.14195) * 100;
		}
		days += 1.0;
	}
	
	/* 변경된 데이터로 다시 그림 */
	[self setNeedsDisplay];
}

/* 현재 날짜 변경 */
- (void)setCurrentDate:(int)year atMonth:(int)month atDay:(int)day {
	[currentDate setYear:year];
	[currentDate setMonth:month];
	[currentDate setDay:day];
}

/* 생일 변경 */
- (void)setBirthDate:(int)year atMonth:(int)month atDay:(int)day {
	[birthDate setYear:year];
	[birthDate setMonth:month];
	[birthDate setDay:day];
	
	NSLog(@"SET BIRTHDAY: %d, %d, %d", year, month, day);
}

/* 사용자가 버튼을 클릭했을 때, 날짜를 변경한다 */
- (void)changeDate:(int)pos {
	NSRange monthRange;
	
	NSTimeInterval newValue = 0;
	NSDate *tempDate = [[NSCalendar currentCalendar] dateFromComponents:currentDate];
	
	if (pos == NAV_PREVMONTH) {
		/* 이전 달로 이동을 위해 이전 달의 날수를 구한다 */
		NSDateComponents *prevMonthDate = [[NSDateComponents alloc] init];
		[prevMonthDate setYear: [currentDate year]];
		[prevMonthDate setMonth: [currentDate month]-1];
		[prevMonthDate setDay:1];
		
		monthRange = [[NSCalendar currentCalendar] rangeOfUnit:NSDayCalendarUnit 
					inUnit:NSMonthCalendarUnit 
					forDate:[[NSCalendar currentCalendar] dateFromComponents:prevMonthDate]];
		[prevMonthDate release];
		
		/* 한달 전으로 이동 */
		newValue = ONEDAY_SECOND * monthRange.length;
		newValue = -newValue;
	}
	else if (pos == NAV_PREVDAY) {
		/* 하루 전으로 이동 */
		newValue = -ONEDAY_SECOND;
	}	
	else if (pos == NAV_NEXTDAY) {
		/* 하루 후로 이동 */
		newValue = ONEDAY_SECOND;
	}	
	else if (pos == NAV_NEXTMONTH) {
		/* 다음 달로 이동을 위해 현재 달의 날수를 구한다 */
		monthRange = [[NSCalendar currentCalendar] rangeOfUnit:NSDayCalendarUnit 
														inUnit:NSMonthCalendarUnit 
													   forDate:[[NSCalendar currentCalendar] dateFromComponents:currentDate]];
		/* 한달 후로 이동 */
		newValue = ONEDAY_SECOND * monthRange.length;
	}

	/* 날짜를 변경하고 변경된 날짜를 구한다. */
	NSDateComponents *newDate = [[NSCalendar currentCalendar] components:(NSYearCalendarUnit | NSMonthCalendarUnit |  NSDayCalendarUnit) 
															fromDate:[tempDate addTimeInterval:newValue]];
	
	/* 현재 날짜를 변경 */
	[currentDate setYear: [newDate year]];
	[currentDate setMonth: [newDate month]];
	[currentDate setDay: [newDate day]];
	
	/* 바이오리듬을 다시 계산하고 출력한다. */
	[self resetBiorhythmData];
}

- (float *)getBiorhythmDayData:(int)day {
	return biorhythmData[day-1];
}

- (NSDateComponents *)currentDate {
	return currentDate;
}
@end
